/*********************************************
This program was produced by the
CodeVisionAVR V1.23.9b Standard
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : MIDI Control
Version : 1.0
Date    : 28.05.2004
Author  : Joerg Engelhardt                 
Company : WE electronics
Comments: 
Ausgabe von MIDI Steuerbefehlen


Chip type           : AT90S2313
Clock frequency     : 4,000000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 32
*********************************************/

#include <90s2313.h>
#include <stdio.h>		// Standard Input/Output functions
#include <delay.h>		// delay_ms()
#include <sleep.h>		// powerdown()

#define UDRE	5		// UART Status Register Bit Definitions

// Prototypes
void TransmitByte( unsigned char data );

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
    GIMSK=0x00;			// Key-IRQ disable
    TransmitByte(0xFB);		// MIDI Continue
    delay_ms(200);
    GIMSK=0xC0;			// Key-IRQ enabled
}

// External Interrupt 1 service routine
interrupt [EXT_INT1] void ext_int1_isr(void)
{
    GIMSK=0x00;			// Key-IRQ disable
    TransmitByte(0xFA);		// MIDI Start
    delay_ms(200);
    GIMSK=0xC0;			// Key-IRQ enabled
}

void main(void)
{
    // Port initialization (all pins input, tristate)
    PORTB=0x00;
    DDRB=0x00;
    PORTD=0x0C;
    DDRD=0x00;
    
    // Timer/Counter initialization (stopped)
    TCCR0=0x00;
    TCNT0=0x00;
    TCCR1A=0x00;
    TCCR1B=0x00;
    TCNT1H=0x00;
    TCNT1L=0x00;
    OCR1H=0x00;
    OCR1L=0x00;
    
    GIMSK=0xC0;		// External Interrupts enable, Mode: Low Level
    MCUCR=0x30;		// Sleep enable, Sleep mode: power down
    GIFR=0xC0;
    
    TIMSK=0x00;		// Timers/Counters Interrupts disabled
    
    // UART initialization
    // Communication Parameters: 8 Data, 1 Stop, No Parity
    // UART Receiver: Off
    // UART Transmitter: On
    // UART Baud rate: 31250 (MIDI)
    UCR=0x08;
    UBRR=0x07;
    
    // Analog Comparator initialization (Off)
    ACSR=0x80;
    
    #asm("sei")			// Global enable interrupts
    
    GIMSK=0x00;			// Key-IRQ disable
    TransmitByte(0xFC);		// MIDI Stop
    delay_ms(300);
    GIMSK=0xC0;			// Key-IRQ enabled

    while (1)
    {
	powerdown();
    };
}

/****************************************************************/
// Sende Routine (AppNote AVR306)
void TransmitByte(unsigned char data)
{
    while(!(USR & (1<<UDRE)));	// wait for empty transmit buffer
    UDR = data;	  		// start transmittion
}
